<?php
class ClassGAPI {
	
	/**
	  * 「Google APIs（https://console.developers.google.com/）」の
	  * 「認証情報」からコールバックのファイルのURLを登録。
	  * JSONをDLして置き換えるか、JSONを書き換える。
	  *
	  * ▼各サービスのPATH
	  * gapi\google-api-php-client\vendor\google\apiclient-services\src\Google\Service
	 **/
	
	public $client;
	
	public function __construct($pathClienJson){
		require_once __DIR__.'/google-api-php-client/vendor/autoload.php';
		
		// ▼「src\Google\Client.php」
		$this->client = new Google_Client();
		
		// 認証情報の読み込み（client_id.json）→jsonの中身を配列で渡しても大丈夫っぽい
		$this->client->setAuthConfig($pathClienJson);
	}
	
	function __destruct(){}
	
	/** Googleクライアントを返す **/
	public function getClient(){
		return $this->client;
	}
	
	/** OAUTHでGoogle認証してトークンを返す **/
	public function gAuth($pathCallback='', $code=''){
		$this->client->setRedirectUri($pathCallback);
		
		// リフレッシュトークン取得に必要
		$this->client->setAccessType('offline');
		$this->client->setApprovalPrompt('force');
		
		if ( strlen($code) == 0 ) {
			// Google認証画面へ
			header('Location: ' . filter_var($this->client->createAuthUrl(), FILTER_SANITIZE_URL));
			exit;
		}
		
		$this->client->authenticate($code);
		return $this->client->getAccessToken();
	}
	
	/** Google認証が未チェック→OAUTHへリダイレクト **/
	public function chkGAuth($pathCallback='', $tokenAccess=false, $isAuth=true){
		$chkGAuth = true;
		
		if ( !is_array($tokenAccess) ) {
			$chkGAuth = false;
		}
		else {
			$this->client->setAccessToken($tokenAccess);
			if ($this->client->isAccessTokenExpired()) {
				// 有効期限切れ
				$chkGAuth = false;
			}
		}
		
		if ( $isAuth && !$chkGAuth ) {
			header('Location: ' . filter_var($pathCallback, FILTER_SANITIZE_URL));
			exit();
		}
		
		return $chkGAuth;
	}
	
	// =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	// ▼各種スコープの設定：apiclient-services\src\Google\Service」の各PHPファイルで「const」されている
	
	/** サーチコンソール **/
	public function setScopeSearchConsole(){
		$this->client->addScope(Google_Service_Webmasters::WEBMASTERS_READONLY);
	}
	
	/** アナリティクス **/
	public function setScopeAnalytics(){
		$this->client->addScope(Google_Service_Analytics::ANALYTICS_READONLY);
	}
	
	/** 個人情報（GooglePeople） **/
	public function setScopePeopel(){
		$this->client->addScope(Google_Service_People::PLUS_LOGIN);
		$this->client->addScope(Google_Service_People::USER_EMAILS_READ);
	}
	
	/** カレンダー **/
	public function setScopeCalendar(){
		$this->client->addScope(Google_Service_Calendar::CALENDAR);
	}
	
	// =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	// ▼各サービスのルーチン
	
	// =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-=
	// ■サーチコンソール
	public function newSearchConsole(){
		$this->setScopeSearchConsole();
		
		// Google_Service_Webmastersを呼び出してSearch Consoleに登録済みのサイトをリストする
		$service = new Google_Service_Webmasters($this->client);
		
		return $service;
	}
	
	/** 取得ルールの基本型 **/
	public function getRuleWithSearchConsole() {
		return array(
		  'rowLimit'   => 5000,
		  'dimensions' => array('query'),
		  'startDate'  => date('Y-m-d', strtotime("-30 day")),
		  'endDate'    => date('Y-m-d'),
		);
	}
	
	/** キーワード一覧の取得 **/
	public function getKwdWithSearchConsole($service, $siteUrl, $prms) {
		// 取得条件をGoogleAPI用に成型
		$postBody = $this->fixPrmBodyWithSearchConsole($prms);
		
		// データ取得用のインスタンス
		$instance = $service->searchanalytics;
		
		// データ取得
		return $instance->query($siteUrl, $postBody, array());
	}
	
	/** リクエストの整形 **/
	public function fixPrmBodyWithSearchConsole($prmBody) {
		
		if ( !is_array($prmBody) ) {
			return $prmBody;
		}
		
		$query = new Google_Service_Webmasters_SearchAnalyticsQueryRequest();
		
		// 必須：期間：YYYY-MM-DD
		$query->setStartDate($this->getValInAry($prmBody, 'startDate', date('Y-m-d', strtotime('-30 day'))));
		$query->setEndDate($this->getValInAry($prmBody, 'endDate', date('Y-m-d', strtotime('-3 day'))));
		
		// データの集計方法
		$query->setAggregationType($this->getValInAry($prmBody, 'aggregationType', 'auto'));
		
		// 配列: 例：array('country', 'device')：country, device, page, query, searchAppearance, ・・・「date」も使える。
		$query->setDimensions($this->getValInAry($prmBody, 'dimensions', array('query')));
		
		// 取得開始：Default is 0. Zero-based index
		$query->setStartRow($this->getValInAry($prmBody, 'startRow', 0));
		
		// 取得数：Valid range is 1–5,000; Default is 1,000
		$query->setRowLimit($this->getValInAry($prmBody, 'rowLimit', 1000));
		
		if ( isset($prmBody['dimensionFilterGroups']) && isset($prmBody['dimensionFilterGroups']['filters']) ) {
			$filtergrp = new Google_Service_Webmasters_ApiDimensionFilterGroup();
			
			$tmpFilter = array();
			
			foreach ( $prmBody['dimensionFilterGroups']['filters'] as $rowFilters ) {
				$filter = new Google_Service_Webmasters_ApiDimensionFilter();
				
				// 対象
				if ( isset($rowFilters['dimension']) ) {
					// country, device, page, query, searchAppearance, ・・・「date」は使えるかもしれない。
					$filter->setDimension($rowFilters['dimension']);
				}
				
				// dimensionの値
				if ( isset($rowFilters['expression']) ) {
					// dimensionでdeviceを使うときは「DESKTOP、MOBILE、TABLET」のいずれか
					$filter->setExpression($rowFilters['expression']);
				}
				
				// dimensionとexpressionの値の関係
				if ( isset($rowFilters['operator']) ) {
					// 含む：contains、等しい：equals、それぞれの否定：notContains、notEquals
					$filter->setOperator($rowFilters['operator']);
				}
				
				// $filtergrp->setFilters(array($filter));
				$tmpFilter[] = $filter;
				
				$filter = null;
				unset($filter);
			}
			
			$filtergrp->setFilters($tmpFilter);
			
			// 配列: 例：array('country', 'device')：country, device, page, query, searchAppearance, ・・・「date」も使える。
			$filtergrp->setGroupType($this->getValInAry($prmBody['dimensionFilterGroups'], 'groupType', 'and'));
			
			$query->setDimensionFilterGroups(array($filtergrp));
		}
		
		if ( isset($prmBody['searchType']) ) {
			// Acceptable values are: image, video, web
			$query->setSearchType($prmBody['searchType']);
		}
		
		return $query;
	}
	
	/** 取得したオブジェクトを配列にする **/
	public function fixgetDataWithSearchConsole($tmpObj) {
		$result = array();
		if ( is_object($tmpObj) ) {
			foreach ( $tmpObj as $rowObj ) {
				$result[] = $this->fixRowDataWithSearchConsole($rowObj);
			}
		}
		
		return $result;
	}
	
	/** 取得したオブジェクトの一つを配列にする **/
	public function fixRowDataWithSearchConsole($tmpObj) {
		return array(
		   'keys'        => $this->getAryWithDay($tmpObj, 'keys',        array()), // 
		   'clicks'      => $this->getAryWithDay($tmpObj, 'clicks',            0), // クリック数・流入数
		   'impressions' => $this->getAryWithDay($tmpObj, 'impressions',       0), // 表示回数
		   'ctr'         => $this->getAryWithDay($tmpObj, 'ctr',               0), // CTR
		   'position'    => $this->getAryWithDay($tmpObj, 'position',          0), // 掲載順位
		);
	}
	
	// =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-=
	// ■アナリティクス
	public function newAnalytics(){
		$this->setScopeAnalytics();
		
		$service = new Google_Service_Analytics($this->client);
		
		return $service;
	}
	
	// =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-=
	// ■個人情報：GooglePeople
	public function newPeople(){
		$this->setScopePeopel();
		
		$service = new Google_Service_People($this->client);
		
		return $service;
	}
	
	// =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-= =-=-+-=-=
	// ■カレンダー
	public function newCalendar(){
		$this->setScopeCalendar();
		
		$service = new Google_Service_Calendar($this->client);
		
		return $service;
	}
	
	// =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	// ▼さぶルーチンず
	
	/** 「配列：$ary」から指定した「キー：$key」の値取得（なければ「初期値：$initVal」を返す） **/
	public function getValInAry($ary, $key, $initVal=false) {
		if ( !is_array($ary) ) {
			return $initVal;
		}
		
		if ( isset($ary[$key]) ) {
			return $ary[$key];
		}
		else {
			return $initVal;
		}
	}
	
	/** 日付から配列作成：キーはunixtime。 **/
	public function getAryWithDay($fstAt, $finAt) {
		$aryDayRank = array();
		
		$i = 1; // 保険：無限ループの回避用
		
		$viewDate = strtotime($fstAt);
		$lastDate = strtotime($finAt);
		
		while ( $viewDate <= $lastDate ) {
			$aryDayRank[$viewDate] = '';
			
			$viewDate += 60*60*24;
			
			if ( 365 <= $i ) {
				break;
			}
			$i++;
		}
		
		return $aryDayRank;
	}
}